@@ -19,8 +19,10 @@
  *  Authors :
  *    Lauris Kaplinski <lauris@ximian.com>
  *    Chema Celorio <chema@ximian.com>
+ *    Andreas J. Guelzow <aguelzow@taliesin.ca>
  *
  *  Copyright (C) 2000-2003 Ximian, Inc. 
+ *  Copyright (C) 2004 Andreas J. Guelzow
  *
  */
 
@@ -44,6 +46,7 @@ static gint gpa_transport_selector_construct (GPAWidget *widget);
 static void gpa_transport_selector_file_button_clicked_cb       (GtkButton *button, GPATransportSelector *ts);
 static void gpa_transport_selector_custom_entry_changed_cb (GtkEntry *entry, GPATransportSelector *ts);
 static void gpa_transport_selector_node_modified_cb (GPANode *node, guint flags, GPATransportSelector *ts);
+static gboolean gpa_transport_selector_check_consistency_real (GPATransportSelector *ts);
 
 static GPAWidgetClass *parent_class;
 
@@ -79,6 +82,8 @@ gpa_transport_selector_class_init (GPATransportSelectorClass *klass)
 
 	gpa_class->construct   = gpa_transport_selector_construct;
 	object_class->finalize = gpa_transport_selector_finalize;
+	klass->check_consistency 
+		= gpa_transport_selector_check_consistency_real;
 }
 
 static void
@@ -90,6 +95,7 @@ gpa_transport_selector_init (GPATransportSelector *ts)
 
 	ts->menu = gtk_option_menu_new ();
 	ts->file_name   = g_strdup ("");
+	ts->file_name_force = FALSE;
 	ts->file_name_label   = gtk_label_new ("");
 	ts->file_button   = gtk_button_new_from_stock (GTK_STOCK_SAVE_AS);
 	ts->custom_entry = gtk_entry_new ();
@@ -130,7 +136,7 @@ gpa_transport_selector_finalize (GObject *object)
 	ts = (GPATransportSelector *) object;
 
 	if (ts->file_selector)
-		gtk_widget_destroy (ts->file_selector);
+		gtk_widget_destroy (GTK_WIDGET(ts->file_selector));
 	ts->file_selector = NULL;
 
 	gpa_transport_selector_disconnect (ts);
@@ -146,72 +152,86 @@ gpa_transport_selector_finalize (GObject *object)
 }
 
 static void
-gpa_transport_selector_file_selector_destroyed_cb (GtkObject *object,
-						   GPATransportSelector *ts)
+gpa_transport_selector_file_selected_cb (GtkFileChooser *dialog,
+					 gint response_id,
+					 GPATransportSelector *ts) 
 {
-	ts->file_selector = NULL;
-}
-
-static void
-gpa_transport_selector_file_selected_cb (GtkWidget *button, GPATransportSelector *ts) 
-{
-	const gchar *selected_filename;
+	char *selected_filename;
 	gchar *filename;
 	gsize bytes_read;
 	gsize bytes_written;
 
-	selected_filename = gtk_file_selection_get_filename 
-		(GTK_FILE_SELECTION (ts->file_selector));
-	
-	filename = g_filename_to_utf8 (selected_filename, -1, 
-                                       &bytes_read, &bytes_written,NULL);
+	if (response_id == GTK_RESPONSE_DELETE_EVENT)
+		ts->file_selector = NULL;
 
+	if (response_id != GTK_RESPONSE_OK || 
+	    ((selected_filename = gtk_file_chooser_get_filename (dialog)) 
+	     == NULL)) {
+		gtk_main_quit ();
+		return;
+	}
+
+	filename = g_filename_to_utf8 (selected_filename, -1,
+				       &bytes_read, &bytes_written,NULL);
+	
 	if (g_file_test (selected_filename, G_FILE_TEST_IS_DIR)) {
-		GtkWidget *dialog;
-		dialog = gtk_message_dialog_new 
+		GtkWidget *warning_dialog;
+		warning_dialog = gtk_message_dialog_new
 			(GTK_WINDOW (ts->file_selector),
 			 GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
 			 GTK_BUTTONS_CLOSE,
 			 _("The specified filename \"%s\" is an existing "
 			   "directory."), filename);
 		
-		g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+		g_signal_connect_swapped (GTK_OBJECT (warning_dialog), 
+					  "response",
 					  G_CALLBACK (gtk_widget_destroy),
-					  GTK_OBJECT (dialog));
-		gtk_widget_show (dialog);
+					  GTK_OBJECT (warning_dialog));
+		gtk_widget_show (warning_dialog);
 		return;
 	}
-
+	
 	if (g_file_test (selected_filename, G_FILE_TEST_EXISTS)) {
-		GtkWidget *dialog;
+		GtkWidget *warning_dialog;
 		gint response;
-
-		dialog = gtk_message_dialog_new 
-			(GTK_WINDOW (ts->file_selector), 
+		
+		warning_dialog = gtk_message_dialog_new
+			(GTK_WINDOW (ts->file_selector),
 			 GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
 			 GTK_BUTTONS_YES_NO,
 			 _("Should the file %s be overwritten?"), filename);
 		
-		response = gtk_dialog_run (GTK_DIALOG (dialog));
-
-		gtk_widget_destroy (dialog);
-
-		if (GTK_RESPONSE_YES != response)
+		response = gtk_dialog_run (GTK_DIALOG (warning_dialog));
+		
+		gtk_widget_destroy (warning_dialog);
+		
+		if (GTK_RESPONSE_YES != response) {
+			ts->file_name_force = FALSE;
 			return;
+		}
+		ts->file_name_force = TRUE;
+	} else {
+		ts->file_name_force = FALSE;
 	}
+	
 	/* FIXME: One of these should be enough... */
-	gpa_node_set_path_value (ts->config, 
-				 "Settings.Output.Job.FileName", 
+	gpa_node_set_path_value (ts->config,
+				 "Settings.Output.Job.FileName",
 				 filename);
-	gpa_node_set_path_value (ts->config, 
-				 "Settings.Transport.Backend.FileName", 
+	gpa_node_set_path_value (ts->config,
+				 "Settings.Transport.Backend.FileName",
 				 filename);
 	gpa_node_set_path_value (ts->node, "FileName", filename);
 	g_free (ts->file_name);
 	ts->file_name = filename;
 	gtk_label_set_text (GTK_LABEL (ts->file_name_label), ts->file_name);
+	
+	g_free (selected_filename);
+	gtk_main_quit ();
 }
 
+
+
 static void
 gpa_transport_selector_file_button_clicked_cb (GtkButton *button, GPATransportSelector *ts)
 {
@@ -221,49 +241,62 @@ gpa_transport_selector_file_button_clicked_cb (GtkButton *button, GPATransportSe
 	
 	/* Create the selector */
    
-	ts->file_selector = gtk_file_selection_new 
-		(_("Please specify the location and filename of the output file:"));
+	ts->file_selector = GTK_FILE_CHOOSER
+		(g_object_new (GTK_TYPE_FILE_CHOOSER_DIALOG,
+			       "action", GTK_FILE_CHOOSER_ACTION_SAVE,
+			       "title", _("Please specify the location and filename of the output file:"),
+			       NULL));
+	gtk_dialog_add_buttons (GTK_DIALOG (ts->file_selector),
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+				NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (ts->file_selector), 
+					 GTK_RESPONSE_OK);
+	/* Filters */
+	{	
+		GtkFileFilter *filter;
+
+		filter = gtk_file_filter_new ();
+		gtk_file_filter_set_name (filter, _("All Files"));
+		gtk_file_filter_add_pattern (filter, "*");
+		gtk_file_chooser_add_filter (ts->file_selector, filter);
+ 
+		filter = gtk_file_filter_new ();
+		gtk_file_filter_set_name (filter, "PDF Files");
+		gtk_file_filter_add_pattern (filter, "*.pdf");
+		gtk_file_chooser_add_filter (ts->file_selector, filter);
+
+		filter = gtk_file_filter_new ();
+		gtk_file_filter_set_name (filter, "Postscript Files");
+		gtk_file_filter_add_pattern (filter, "*.ps");
+		gtk_file_chooser_add_filter (ts->file_selector, filter);
+	}
+
+	gtk_file_chooser_unselect_all (ts->file_selector);
 	
 	filename = g_filename_from_utf8 (ts->file_name, -1,
 					 &bytes_read, &bytes_written, NULL);
 	if (filename != NULL) {
-		gtk_file_selection_set_filename 
-			(GTK_FILE_SELECTION(ts->file_selector),
-			 filename);
+		gtk_file_chooser_set_current_name (ts->file_selector, 
+						   filename);
 		g_free (filename);
 	}
    
-	g_signal_connect 
-		(GTK_OBJECT(GTK_FILE_SELECTION(ts->file_selector)->ok_button),
-		 "clicked",
-		 G_CALLBACK(gpa_transport_selector_file_selected_cb),
-		 (gpointer) ts);
-	g_signal_connect 
-		(GTK_OBJECT(ts->file_selector),
-		 "destroy",
-		 G_CALLBACK(gpa_transport_selector_file_selector_destroyed_cb),
-		 (gpointer) ts);
-   			   
-	/* Ensure that the dialog box is destroyed when the user clicks a button. */
+	g_signal_connect (ts->file_selector, "response",
+			  G_CALLBACK (gpa_transport_selector_file_selected_cb),
+			  (gpointer) ts);
 	
-	g_signal_connect_swapped 
-		(GTK_OBJECT(GTK_FILE_SELECTION(ts->file_selector)
-			    ->ok_button),
-		 "clicked",
-		 G_CALLBACK(gtk_widget_destroy), 
-		 (gpointer) ts->file_selector); 
-	
-	g_signal_connect_swapped 
-		(GTK_OBJECT(GTK_FILE_SELECTION(ts->file_selector)
-			    ->cancel_button),
-		 "clicked",
-		 G_CALLBACK(gtk_widget_destroy),
-		 (gpointer) ts->file_selector); 
-    
 	gtk_window_set_modal(GTK_WINDOW (ts->file_selector), TRUE);
 	/* Display that dialog */
-	gtk_widget_show (ts->file_selector);
+	gtk_widget_show_all (GTK_WIDGET(ts->file_selector));
 	
+	gtk_grab_add (GTK_WIDGET(ts->file_selector));
+	gtk_main ();
+
+	if (ts->file_selector != NULL) {
+		gtk_widget_destroy (GTK_WIDGET (ts->file_selector));
+		ts->file_selector = NULL;
+	}
 }
 
 static void
@@ -440,3 +473,95 @@ gpa_transport_selector_construct (GPAWidget *gpaw)
 	return TRUE;
 }
 
+static gboolean 
+gpa_transport_selector_check_consistency_real (GPATransportSelector *ts)
+{
+	gchar *backend;
+	gchar *selected_filename;
+	gsize bytes_read, bytes_written;
+
+	backend  = gpa_node_get_path_value 
+		(ts->config, "Settings.Transport.Backend");
+
+	if (backend == NULL)
+		return TRUE;
+
+	if (strcmp (backend, "file") != 0)
+		return TRUE;
+	
+	if (ts->file_name_force)
+		return TRUE;
+
+	g_return_val_if_fail (ts->file_name != NULL, FALSE);
+
+	selected_filename = g_filename_from_utf8 (ts->file_name, -1,
+						  &bytes_read, &bytes_written,
+						  NULL);
+
+	g_return_val_if_fail (selected_filename != NULL, FALSE);
+
+	if (g_file_test (selected_filename, G_FILE_TEST_IS_DIR)) {
+		GtkWidget *dialog;
+		GtkWidget *window = ts->file_name_label;
+		
+		while ((window != NULL) && !(GTK_IS_WINDOW (window)))
+			window = gtk_widget_get_parent (window);
+		
+		dialog = gtk_message_dialog_new 
+			(GTK_WINDOW(window),
+			 GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
+			 GTK_BUTTONS_CLOSE,
+			 _("The specified filename \"%s\" is an existing "
+			   "directory."), ts->file_name);
+		
+		gtk_dialog_run (GTK_DIALOG(dialog));
+		gtk_widget_destroy (dialog);
+
+		g_free (selected_filename);
+		return FALSE;
+	}
+
+	if (g_file_test (selected_filename, G_FILE_TEST_EXISTS)) {
+		GtkWidget *dialog;
+		GtkWidget *window = ts->file_name_label;
+		gint response;
+
+		while ((window != NULL) && !(GTK_IS_WINDOW (window)))
+			window = gtk_widget_get_parent (window);
+
+		dialog = gtk_message_dialog_new 
+			(GTK_WINDOW(window), 
+			 GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
+			 GTK_BUTTONS_YES_NO,
+			 _("Should the file %s be overwritten?"), 
+			 ts->file_name);
+		
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+		gtk_widget_destroy (dialog);
+
+		if (GTK_RESPONSE_YES != response) {
+			ts->file_name_force = FALSE;
+			g_free (selected_filename);
+			return FALSE;
+		}
+		ts->file_name_force = TRUE;
+	} else {
+		ts->file_name_force = FALSE;
+	}
+
+	g_free (selected_filename);
+	return TRUE;
+}
+
+gboolean 
+gpa_transport_selector_check_consistency (GPATransportSelector *ts)
+{
+	GPATransportSelectorClass *klass 
+		= GPA_TRANSPORT_SELECTOR_GET_CLASS (ts);
+
+	if (klass->check_consistency == NULL)
+		return TRUE;
+
+	return (klass->check_consistency (ts));
+}
